@@ -1,14 +1,13 @@
 package CPAN::Search::Lite::Util;
 use strict;
 use warnings;
-use Sort::Versions;
-our $VERSION = 0.74;
+our $VERSION = 0.76;
 
 use base qw(Exporter);
 our (@EXPORT_OK, %chaps, %chaps_rev, $repositories, %modes,
      $table_id, $query_info, $mode_info, $full_id, $tt2_pages);
 @EXPORT_OK = qw(%chaps %chaps_rev $repositories $tt2_pages %modes
-                vcmp $table_id $query_info $mode_info $full_id);
+                vcmp $table_id $query_info $mode_info $full_id has_data);
 
 make_ids();
 
@@ -91,30 +90,30 @@ $query_info = { module => {mode => 'module', type => 'name'},
 
 
 $repositories = {
-                 1 => {
-                       alias => 'crazy56',
-                       LOCATION => 
-                       'http://crazyinsomniac.perlmonk.org/perl/ppm',
-                       SUMMARYFILE  => 'summary.ppm',
-                       browse => 'http://crazyinsomniac.perlmonk.org/perl/ppm',
-                       desc => 'crazyinsomniac Perl 5.6 repository',
-                       build => '6xx',
-                       PerlV         => 5.6,
-                      },
-                 2 => {
-                       alias => 'crazy58',
-                       LOCATION  => 
-                       'http://crazyinsomniac.perlmonk.org/perl/ppm/5.8',
-                       SUMMARYFILE  => 'summary.ppm',
-                       browse => 'http://crazyinsomniac.perlmonk.org/perl/ppm/5.8',
-                       desc => 'crazyinsomniac Perl 5.8 repository',
-                       build => '8xx',
-                       PerlV         => 5.8,
-                      },
+#                 1 => {
+#                       alias => 'crazy56',
+#                       LOCATION => 
+#                       'http://crazyinsomniac.perlmonk.org/perl/ppm',
+#                       SUMMARYFILE  => 'summary.ppm',
+#                       browse => 'http://crazyinsomniac.perlmonk.org/perl/ppm',
+#                       desc => 'crazyinsomniac Perl 5.6 repository',
+#                       build => '6xx',
+#                       PerlV         => 5.6,
+#                      },
+#                 2 => {
+#                       alias => 'crazy58',
+#                       LOCATION  => 
+#                       'http://crazyinsomniac.perlmonk.org/perl/ppm/5.8',
+#                       SUMMARYFILE  => 'summary.ppm',
+#                       browse => 'http://crazyinsomniac.perlmonk.org/perl/ppm/5.8',
+#                       desc => 'crazyinsomniac Perl 5.8 repository',
+#                       build => '8xx',
+#                       PerlV         => 5.8,
+#                      },
                  3 => {
                        alias => 'uwinnipeg56',
                        LOCATION  => 
-                       'http://theoryx5.uwinnipeg.ca/cgi-bin/ppmserver?urn:/PPMServer',
+                       'http://theoryx5.uwinnipeg.ca/ppmpackages/',
                        SUMMARYFILE  => 'fetch_summary',
                        browse => 'http://theoryx5.uwinnipeg.ca/ppmpackages',
                        desc => 'uwinnipeg Perl 5.6 repository',
@@ -124,7 +123,7 @@ $repositories = {
                  4 => {
                        alias => 'uwinnipeg58',
                        LOCATION  => 
-                       'http://theoryx5.uwinnipeg.ca/cgi-bin/ppmserver?urn:/PPMServer58',
+                       'http://theoryx5.uwinnipeg.ca/ppms/',
                        SUMMARYFILE  => 'fetch_summary',
                        browse => 'http://theoryx5.uwinnipeg.ca/ppms',
                        desc => 'uwinnipeg Perl 5.8 repository',
@@ -134,7 +133,7 @@ $repositories = {
                  5 => {
                        alias => 'AS58',
                        LOCATION  => 
-                       'http://ppm.ActiveState.com/cgibin/PPM/ppmserver-5.8-windows.pl?urn:/PPMServer',
+                       'http://ppm.activestate.com/PPMPackages/5.8-windows',
                        SUMMARYFILE  => 'fetch_summary',
                        PerlV         => 5.8,
                        browse => 'http://ppm.activestate.com/BuildStatus/5.8-A.html',
@@ -144,7 +143,7 @@ $repositories = {
                  6 => {
                        alias => 'AS56',
                        LOCATION  => 
-                       'http://ppm.activestate.com/cgibin/PPM/ppmserver.pl?urn:/PPMServer',
+                       'http://ppm.activestate.com/PPMPackages/5.6',
                        SUMMARYFILE  => 'fetch_summary',
                        browse => 'http://ppm.activestate.com/BuildStatus/5.6-A.html',
                        desc => 'ActiveState default Perl 5.6 repository',
@@ -154,7 +153,7 @@ $repositories = {
                  7 => {
                        alias => 'bribes56',
                        LOCATION  => 
-                       'http://www.bribes.org/cgi-bin',
+                       'http://www.bribes.org/perl/ppm/',
                        SUMMARYFILE  => 'summary.cgi',
                        browse => 'http://www.bribes.org/perl/ppm',
                        desc => 'www.bribes.org Perl 5.6 repository',
@@ -164,7 +163,7 @@ $repositories = {
                  8 => {
                        alias => 'bribes58',
                        LOCATION  => 
-                       'http://www.bribes.org/cgi-bin',
+                       'http://www.bribes.org/perl/ppm/',
                        SUMMARYFILE  => 'summary.cgi',
                        browse => 'http://www.bribes.org/perl/ppm',
                        desc => 'www.bribes.org Perl 5.8 repository',
@@ -183,14 +182,127 @@ sub make_ids {
 #    $full_id->{chapterid} = 'chaps.chapterid';
 }
 
-my $num_re = qr{^0*\.\d+$};
+#my $num_re = qr{^0*\.\d+$};
+#sub vcmp {
+#    my ($v1, $v2) = @_;
+#    return unless (defined $v1 and defined $v2);
+#    if ($v1 =~ /$num_re/ and $v2 =~ /$num_re/) {
+#        return $v1 <=> $v2;
+#    }
+#    return Sort::Versions::versioncmp($v1, $v2);
+#}
+
+sub has_data {
+  my $data  = shift;
+  return unless (defined $data and ref($data) eq 'HASH');
+  return (scalar keys %$data > 0) ? 1 : 0;
+}
+
+
 sub vcmp {
-    my ($v1, $v2) = @_;
-    return unless (defined $v1 and defined $v2);
-    if ($v1 =~ /$num_re/ and $v2 =~ /$num_re/) {
-        return $v1 <=> $v2;
+  my ($v1, $v2) = @_;
+  return CPAN::Search::Lite::Version->vcmp($v1, $v2);
+}
+
+
+# This is borrowed essentially verbatim from CPAN::Version
+# It's included here so as to not demand a CPAN.pm upgrade
+
+package CPAN::Search::Lite::Version;
+
+use strict;
+use vars qw($VERSION);
+$VERSION = 0.74;
+
+# CPAN::Version::vcmp courtesy Jost Krieger
+sub vcmp {
+  my ($self, $l, $r) = @_;
+  local($^W) = 0;
+
+  return 0 if $l eq $r; # short circuit for quicker success
+
+  for ($l, $r) {
+      next unless tr/.// > 1;
+      s/^v?/v/;
+      1 while s/\.0+(\d)/.$1/;
+  }
+  if ($l =~ /^v/ <=> $r =~ /^v/) {
+      for ($l, $r) {
+          next if /^v/;
+          $_ = $self->float2vv($_);
+      }
+  }
+
+  return (
+          ($l ne "undef") <=> ($r ne "undef") ||
+          (
+           $] >= 5.006 &&
+           $l =~ /^v/ &&
+           $r =~ /^v/ &&
+           $self->vstring($l) cmp $self->vstring($r)
+          ) ||
+          $l <=> $r ||
+          $l cmp $r
+         );
+}
+
+sub vgt {
+  my ($self, $l, $r) = @_;
+  $self->vcmp($l, $r) > 0;
+}
+
+sub vlt {
+  my ($self, $l, $r) = @_;
+  0 + ($self->vcmp($l, $r) < 0);
+}
+
+sub vstring {
+  my ($self, $n) = @_;
+  $n =~ s/^v// 
+    or die "CPAN::Search::Lite::Version::vstring() called with invalid arg [$n]";
+  pack "U*", split /\./, $n;
+}
+
+# vv => visible vstring
+sub float2vv {
+    my ($self, $n) = @_;
+    my ($rev) = int($n);
+    $rev ||= 0;
+    my ($mantissa) = $n =~ /\.(\d{1,12})/; # limit to 12 digits to limit
+                                          # architecture influence
+    $mantissa ||= 0;
+    $mantissa .= "0" while length($mantissa)%3;
+    my $ret = "v" . $rev;
+    while ($mantissa) {
+        $mantissa =~ s/(\d{1,3})// or
+            die "Panic: length>0 but not a digit? mantissa[$mantissa]";
+        $ret .= ".".int($1);
     }
-    return Sort::Versions::versioncmp($v1, $v2);
+    # warn "n[$n]ret[$ret]";
+    $ret;
+}
+
+sub readable {
+  my($self,$n) = @_;
+  $n =~ /^([\w\-\+\.]+)/;
+
+  return $1 if defined $1 && length($1)>0;
+  # if the first user reaches version v43, he will be treated as "+".
+  # We'll have to decide about a new rule here then, depending on what
+  # will be the prevailing versioning behavior then.
+
+  if ($] < 5.006) { # or whenever v-strings were introduced
+    # we get them wrong anyway, whatever we do, because 5.005 will
+    # have already interpreted 0.2.4 to be "0.24". So even if he
+    # indexer sends us something like "v0.2.4" we compare wrongly.
+
+    # And if they say v1.2, then the old perl takes it as "v12"
+
+    warn("Suspicious version string seen [$n]\n");
+    return $n;
+  }
+  my $better = sprintf "v%vd", $n;
+  return $better;
 }
 
 1;
